/*
 * configmotif.C
 *
 * Motif interface for the "Config" class
 *
 * Vincent Mallet 1997, 1999
 */

// $Id: configmotif.C,v 1.2 1999/04/20 21:52:58 vmallet Exp $

/* ViMmail - a small mail reader
 * Copyright (C) 1997, 1999 Vincent Mallet - vmallet@enst.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>

#include "config.H"
#include "configmotif.H"

#include "application.H"  // pour pouvoir faire le feedback
#include "vimmail.H"      // et redemander a l'appli de regler ses settings


#define FREE(string) if ((string)) free((string)); (string) = NULL


// Creator
void ConfigMotif:: make(Widget _parent)
{
  widget = 0;
  parent = _parent;
}


void ConfigMotif:: create_dialog(void)
{
  Widget button, frame, row, temp;
  
  widget = XmCreateFormDialog(parent, "ConfigDialog", NULL, 0);
  
  // save, cancel, default
  row = XtVaCreateManagedWidget("ConfigRow", xmRowColumnWidgetClass, widget, NULL);
  
  button = XmCreatePushButton(row, "ConfigOk", NULL, 0);
  XtVaSetValues(button, XmNalignment, XmALIGNMENT_CENTER, NULL); // seul moyen de le faire marcher!
  XtAddCallback(button, XmNactivateCallback, ok_cb, (XtPointer) this);
  XtManageChild(button);
  
  button = XmCreatePushButton(row, "ConfigCancel", NULL, 0);
  XtVaSetValues(button, XmNalignment, XmALIGNMENT_CENTER, NULL);
  XtAddCallback(button, XmNactivateCallback, cancel_cb, (XtPointer) this);
  XtManageChild(button);
  
  button = XmCreatePushButton(row, "ConfigDefault", NULL, 0);
  XtVaSetValues(button, XmNalignment, XmALIGNMENT_CENTER, NULL);
  XtAddCallback(button, XmNactivateCallback, default_cb, (XtPointer) this);
  XtManageChild(button);
  
  // general options
  temp = XtVaCreateManagedWidget("ConfigFrame1", xmFrameWidgetClass, widget, NULL);
  XtVaCreateManagedWidget("ConfigTitle1", xmLabelWidgetClass, temp, NULL);

  temp = XtVaCreateManagedWidget("ConfigForm1", xmFormWidgetClass, temp, NULL);
  row = XtVaCreateManagedWidget("ConfigRow1", xmRowColumnWidgetClass, temp, NULL);
  b1 = XtVaCreateManagedWidget("ConfigB1", xmToggleButtonWidgetClass, row, NULL);
  b2 = XtVaCreateManagedWidget("ConfigB2", xmToggleButtonWidgetClass, row, NULL);
  b3 = XtVaCreateManagedWidget("ConfigB3", xmToggleButtonWidgetClass, row, NULL);
  b4 = XtVaCreateManagedWidget("ConfigB4", xmToggleButtonWidgetClass, row, NULL);


  XtVaCreateManagedWidget("ConfigB5title", xmLabelWidgetClass, temp, NULL);
  b5 = XtVaCreateManagedWidget("ConfigB5", xmTextFieldWidgetClass, temp, NULL);
  XtAddCallback(b5, XmNmodifyVerifyCallback, checknum_cb, (XtPointer) this);
  
  
  // colors
  frame = XtVaCreateManagedWidget("ConfigFrame2", xmFrameWidgetClass, widget, NULL);
  XtVaCreateManagedWidget("ConfigTitle2", xmLabelWidgetClass, frame, NULL);
  
  row = XtVaCreateManagedWidget("ConfigForm2", xmFormWidgetClass, frame, NULL);
  
  XtVaCreateManagedWidget("ConfigC1title", xmLabelWidgetClass, row, NULL);
  t1 = XtVaCreateManagedWidget("ConfigC1", xmTextFieldWidgetClass, row, NULL);
  
  XtVaCreateManagedWidget("ConfigC2title", xmLabelWidgetClass, row, NULL);
  t2 = XtVaCreateManagedWidget("ConfigC2", xmTextFieldWidgetClass, row, NULL);
  
  XtVaCreateManagedWidget("ConfigC3title", xmLabelWidgetClass, row, NULL);
  t3 = XtVaCreateManagedWidget("ConfigC3", xmTextFieldWidgetClass, row, NULL);
  
  XtVaCreateManagedWidget("ConfigC4title", xmLabelWidgetClass, row, NULL);
  t4 = XtVaCreateManagedWidget("ConfigC4", xmTextFieldWidgetClass, row, NULL);    


  // paths
  frame = XtVaCreateManagedWidget("ConfigFrame3", xmFrameWidgetClass, widget, NULL);
  XtVaCreateManagedWidget("ConfigTitle3", xmLabelWidgetClass, frame, NULL);
  
  row = XtVaCreateManagedWidget("ConfigForm3", xmFormWidgetClass, frame, NULL);
  
  XtVaCreateManagedWidget("ConfigP1title", xmLabelWidgetClass, row, NULL);
  p1 = XtVaCreateManagedWidget("ConfigP1", xmTextFieldWidgetClass, row, NULL);

  XtVaCreateManagedWidget("ConfigP2title", xmLabelWidgetClass, row, NULL);
  p2 = XtVaCreateManagedWidget("ConfigP2", xmTextFieldWidgetClass, row, NULL);

  XtVaCreateManagedWidget("ConfigP3title", xmLabelWidgetClass, row, NULL);
  p3 = XtVaCreateManagedWidget("ConfigP3", xmTextFieldWidgetClass, row, NULL);





}


void ConfigMotif:: set_dialog_values(void)
{
  char temp[64];
  XmTextFieldSetString(t1, header_color_string);
  XmTextFieldSetString(t2, header_high_color_string);
  XmTextFieldSetString(t3, text_color_string);
  XmTextFieldSetString(t4, quote_color_string);
  
  XmToggleButtonSetState(b1, ask_before_quit, False);
  XmToggleButtonSetState(b2, toolbar_visible, False);
  XmToggleButtonSetState(b3, popup_on_startup, False);
  XmToggleButtonSetState(b4, show_last_message, False);

  sprintf(temp, "%d", tab_expand);
  XmTextFieldSetString(b5, temp);

  XmTextFieldSetString(p1, home_path);
  XmTextFieldSetString(p2, spool_path);
}


void ConfigMotif:: prompt_config(void)
{
  if (!widget)
    create_dialog();

  set_dialog_values();

  XtManageChild(widget);
  XtPopup(XtParent(widget), XtGrabNone);
}




void ConfigMotif:: update_config(void)
{
  char *text;

  ask_before_quit   = XmToggleButtonGetState(b1);
  toolbar_visible   = XmToggleButtonGetState(b2);
  popup_on_startup  = XmToggleButtonGetState(b3);
  show_last_message = XmToggleButtonGetState(b4);

  text = XmTextFieldGetString(b5);
  if (*text) 
    tab_expand = atoi(text);
  XtFree(text);

  FREE(header_color_string);
  text = XmTextFieldGetString(t1);
  header_color_string = strdup(text);
  XtFree(text);

  FREE(header_high_color_string);
  text = XmTextFieldGetString(t2);
  header_high_color_string = strdup(text);
  XtFree(text);

  FREE(text_color_string);
  text = XmTextFieldGetString(t3);
  text_color_string = strdup(text);
  XtFree(text);

  FREE(quote_color_string);
  text = XmTextFieldGetString(t4);
  quote_color_string = strdup(text);
  XtFree(text);

  FREE(spool_path);
  text = XmTextFieldGetString(p2);
  spool_path = strdup(text);
  XtFree(text);
}


void ConfigMotif:: ok_cb(Widget w, XtPointer client_data, XtPointer cbs)
{
  ConfigMotif * mythis = (ConfigMotif *) client_data;
  w = w; cbs = cbs;

  mythis->update_config();
  mythis->save_config();

  vimmail.reconfigure();
  
  XtDestroyWidget(mythis->widget);
  mythis->widget = 0;
}

void ConfigMotif:: cancel_cb(Widget w, XtPointer client_data, XtPointer cbs)
{
  ConfigMotif * mythis = (ConfigMotif *) client_data;
  w = w; cbs = cbs;

  XtDestroyWidget(mythis->widget);
  mythis->widget = 0;
}

void ConfigMotif:: default_cb(Widget w, XtPointer client_data, XtPointer cbs)
{
  ConfigMotif * mythis = (ConfigMotif *) client_data;
  w = w; cbs = cbs;

  mythis->set_default();
  mythis->set_dialog_values();
}


void ConfigMotif:: checknum_cb(Widget w, XtPointer client_data, XtPointer tvcbs)
{
  XmTextVerifyCallbackStruct *cbs = (XmTextVerifyCallbackStruct *)tvcbs;
  int len = XmTextFieldGetLastPosition(w);

  client_data = client_data;

  if (cbs->startPos < cbs->currInsert)  // backspace
    return;

  if (len == 3) {
    cbs->doit = False;
    return;
  }

  if (len + cbs->text->length > 3) {
    cbs->text->ptr[3 - len] = 0;
    cbs->text->length = strlen(cbs->text->ptr);
  }

  for (len = 0; len < cbs->text->length; len++) {
    if (!isdigit(cbs->text->ptr[len])) {
      for (int i = len; (i+1) < cbs->text->length; i++)
	cbs->text->ptr[i] = cbs->text->ptr[i+1];
      cbs->text->length--;
      len--;
    }
  }
  
  if (cbs->text->length == 0)
    cbs->doit = False;
}

