/*
 * interface.C
 *
 * The Motif interface for the Application
 *
 * Vincent Mallet 1997, 1999
 */

// $Id: interface.C,v 1.2 1999/04/20 21:58:14 vmallet Exp $

/* ViMmail - a small mail reader
 * Copyright (C) 1997, 1999 Vincent Mallet - vmallet@enst.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <X11/Xmu/Editres.h>
#include <X11/cursorfont.h>
#include <X11/bitmaps/mailfull>
#include <X11/bitmaps/mailempty>
#include <Xm/Form.h>
#include <Xm/List.h>
#include <Xm/PanedW.h>
// #include <Xm/Scale.h>

#include "vimmail.H"

#include "VimDebug.H"

#include "motifutil.H"
#include "menus.H"
#include "interface.H"
#include "fileselect.H"
#include "filelist.H"
#include "msglist.H"
#include "eyes.H"
#include "application.H"
#include "mailarea.H"
#include "popup.H"

#include "pushbuttons.h"
#include "cluepushbutton.h"
#include "Pixmaps/biff_empty.xpm"
#include "Pixmaps/biff_full.xpm"


#include "resources.inc"        // Default resources


enum {  // Icon types 
  IconUndefined,
  IconEmpty,
  IconFull
};


// Constructor
Interface:: Interface(void) : toplevel(0), real_toplevel(0), main_window(0), current_icon(IconUndefined)
{
}

// Actually create this interface
void Interface:: make(int argc, char **argv)
{
  int n;
  Widget form;

  Widget paned; // a l'arrache, comme d'hab

  XtSetLanguageProc(NULL, (XtLanguageProc) NULL, NULL);

  //  prog_name = (char *) XtNewString(argv[0]);
  n=0;

  //  XtSetArg(args[n], XmNiconName, TMxICON ); n++;

  toplevel = XtVaAppInitialize(&context, VM_CLASSNAME, NULL, 0, &argc, argv, ViMresources, NULL);

  XtVaSetValues(toplevel, XmNtitle, VM_CLASSNAME, NULL, NULL);

/*
  icon_full = XCreatePixmapFromBitmapData(XtDisplay(toplevel), RootWindowOfScreen(XtScreen(toplevel)),
					  (char *) mailfull_bits, mailfull_width, mailfull_height, 1, 0, 1);

  icon_empty = XCreatePixmapFromBitmapData(XtDisplay(toplevel), RootWindowOfScreen(XtScreen(toplevel)),
					   (char *) mailempty_bits, mailempty_width, mailempty_height, 1, 0, 1);

  set_empty();
*/

  XtAddEventHandler(toplevel, (EventMask) 0, True, (XtEventHandler) _XEditResCheckMessages, 0);

  main_window = XtVaCreateManagedWidget("MainWindow2", xmMainWindowWidgetClass, toplevel, NULL);
  togglebutton = XmCreatePushButton(main_window, "PopToggleButton", NULL, 0);
  set_empty();
  XtAddCallback(togglebutton, XmNactivateCallback, pop_handler, (XtPointer) this);
  XtManageChild(togglebutton);

  real_toplevel = XtVaCreatePopupShell("RealTopLevel", topLevelShellWidgetClass, GetTopShell(toplevel), NULL);
  XtVaSetValues(real_toplevel, XmNtitle, VM_CLASSNAME, NULL, NULL);
  XtAddEventHandler(real_toplevel, StructureNotifyMask, False, EventHandler, (XtPointer) this);
  popped = 0;


  main_window = XtVaCreateManagedWidget("MainWindow", xmMainWindowWidgetClass, real_toplevel, NULL);

  display     = XtDisplay(main_window);

  Initialisation_clue(main_window);   // pour les CluePushButtons de Thierry

  fileselect  . make(main_window, ALWAYS_NEW_DIALOG);
  menus       . make(main_window);
  about       . make(main_window);
  form        = XtVaCreateManagedWidget("form", xmFormWidgetClass, main_window, NULL);
  eyes        . make(form);
  toolbar     . make(form);
  paned       = XtVaCreateManagedWidget("paned", xmPanedWindowWidgetClass, form, NULL);
  horzpane    . make(paned);
  filelist    . make(horzpane.widget);
  msglist     . make(horzpane.widget);
  mailarea    . make(paned);
  mailpopup   . make(mailarea.widget);
  mailarea    . set_rightbutton_callback(mailpopup.popup_cb, &mailpopup);
  horzpane    . set_objects(filelist.widget, msglist.widget);

  //  scale    = new Scale(horzpane->widget);
  //  textarea = new TextArea(paned);
  //  scale->set_object(filelist->widget);
  
  TurnOffSashTraversal(paned);

  XmMainWindowSetAreas(main_window, menus.widget, NULL, NULL, NULL, form);

  not_configured = 1;    // we're not configured yet !
}




// Start the Motif interface
void Interface:: run_till_death(void)
{
  reconfigure();

  // met le bazard en route
  XtRealizeWidget(toplevel);

  if (vimmail.config.popup_on_startup)
    popup();

  // et c'est parti.
  XtAppMainLoop(context);
}




void Interface:: pop_handler(Widget w, XtPointer client_data, XtPointer cbs)
{
  Interface * mythis = (Interface *) client_data;
  w = w; cbs = cbs;

  if (mythis->popped)
    mythis->popdown();
  else
    mythis->popup();
}






void Interface:: EventHandler(Widget w, XtPointer client_data, XEvent *event, char *z)
{
  Interface * mythis = (Interface *) client_data;

  w = w;
  z = z;

  switch (event->type) {
  case ConfigureNotify:
//     printf("ConfigureNotify\n");
    if (mythis->not_configured)
      mythis->configure();
    //    mythis->scale->configure();
    break;
  case DestroyNotify:
//     printf("DestroyNotify\n");
    break;
  default:
//     printf("  (bah, autre notify\n");
    break;
  }
}


// Turn the icon into the 'empty' sign
void Interface:: set_empty(void)
{
  if (current_icon != IconEmpty) {
    XtVaSetValues(togglebutton, XmNbackground, WhitePixel(XtDisplay(togglebutton), DefaultScreen(XtDisplay(togglebutton))), NULL);
    CreatePixmapButton(togglebutton, biff_empty, biff_empty, NULL);
    
    current_icon = IconEmpty;
  }
}

// Turn the icon into the 'Full' sign
void Interface:: set_full(void)
{
  if (current_icon != IconFull) {
    XtVaSetValues(togglebutton, XmNbackground, BlackPixel(XtDisplay(togglebutton), DefaultScreen(XtDisplay(togglebutton))), NULL);
    CreatePixmapButton(togglebutton, biff_full, biff_full, NULL);

    current_icon = IconFull;
  }
}

// configure quelques elements de l'interface
// dont les parametres ne peuvent etre connus 
// qu'apres le premier map de la workwindow
void Interface:: configure(void)
{
  horzpane.nice_cursor(XC_sb_h_double_arrow);
  filelist.nice_cursor(XC_hand1);
  msglist.nice_cursor(XC_hand2);
  mailarea.nice_cursor(XC_xterm);
  //  textarea->nice_cursor(XC_xterm);
  //  scale->nice_cursor(XC_sb_h_double_arrow);

  eyes.start_eyes();

  not_configured = 0;  // do this only once
}


// reconfigure quelques elements en reponse
// a un changement de parametres utilisateurs
// (apres Alt+C par exemple)
void Interface:: reconfigure(void)
{
  toolbar.set_visible(vimmail.config.toolbar_visible);
  mailarea.set_config(vimmail.config.header_color_string,
		      vimmail.config.header_high_color_string,
		      vimmail.config.quote_color_string,
		      vimmail.config.text_color_string,
		      vimmail.config.tab_expand);
}


// pop UP the workwindow
void Interface:: popup(void)
{
  XtPopup(real_toplevel, XtGrabNone);
  popped = 1;
}


// pop DOWN the workwindow
void Interface:: popdown(void)
{
  XtPopdown(real_toplevel);
  popped = 0;
}







