/*
 * menus.C
 *
 * The Menu Bar
 *
 * Vincent Mallet 1997, 1999
 */

// $Id: menus.C,v 1.3 1999/04/20 22:23:25 vmallet Exp $

/* ViMmail - a small mail reader
 * Copyright (C) 1997, 1999 Vincent Mallet - vmallet@enst.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <Xm/CascadeB.h>

#include "vimmail.H"

#include "menus.H"
#include "application.H"
#include "fileselect.H"
#include "motifutil.H"


enum { /* item numbers of File menu */
  ItemFileOpen,
  ItemFileClose,
  ItemFileCloseAll,
  ItemFilePopdown,
  ItemFileQuit};

enum { /* item numbers of Erase submenu */
  ItemEraseClean,
  ItemEraseCleanAll};

enum { /* item numbers of Actions menu */
  ItemActionRescanFile,
  ItemActionRescanAll,
  ItemActionSee};

enum { /* item numbers of Options menu */
  ItemOptionConfigure,
  ItemOptionLocalConf,
  ItemOptionInstall};

enum { /* item numbers of Help menu */
  ItemHelpHelp,
  ItemHelpStory,
  ItemHelpAbout};


void Menus:: empty_handler(Widget w, XtPointer client_data, XtPointer cbs) {w = w; client_data = cbs;}

void Menus:: file_handler(Widget w, XtPointer client_data, XtPointer cbs)
{
  w = w; cbs = cbs;

  switch( (int) client_data) {
  case ItemFileOpen:
    vimmail.interface.fileselect.show_dialog();
    break;
  case ItemFilePopdown:
    vimmail.interface.popdown();
    break;
  case ItemFileQuit:
    quit();
    //    vimmail->end_app(); /* on nettoie tout (si necessaire) */
    break;
  }
}


void Menus:: actions_handler(Widget w, XtPointer client_data, XtPointer cbs)
{
  w = w; cbs = cbs;

  switch ((int) client_data) {
  case ItemActionRescanFile:
    vimmail.rescan_message_file();
    break;
  case ItemActionRescanAll:
    vimmail.rescan_all_message_files();
    break;
  }
}

void Menus:: options_handler(Widget w, XtPointer client_data, XtPointer cbs)
{
  w = w; cbs = cbs;

  switch((int) client_data) {
  case ItemOptionConfigure:
     vimmail.config.prompt_config();
    break;
  }
}

void Menus:: erase_handler(Widget w, XtPointer client_data, XtPointer cbs)
{
  w = w;
  client_data = cbs;
}

void Menus:: help_handler(Widget w, XtPointer client_data, XtPointer cbs)
{
  w = w; cbs = cbs;

  switch( (int) client_data) {
  case ItemHelpAbout:
    vimmail.interface.about.show_dialog();
    break;
  }
}










Menus:: Menus(void)
{
  widget = 0;
}

void Menus:: make(Widget main_w)
{
  Widget menu_bar, menu, smenu;
  Widget button;

  main_window = main_w;

  //about = new About(main_w);

  menu_bar = XmVaCreateSimpleMenuBar(main_w, "MenuBar",
				     XmVaCASCADEBUTTON, NULL, 0,
				     XmVaCASCADEBUTTON, NULL, 0,
				     XmVaCASCADEBUTTON, NULL, 0, NULL);
  XtManageChild(menu_bar);

  menu = XmVaCreateSimplePulldownMenu(menu_bar, "FileMenu",
				      0, file_handler,
				      XmVaPUSHBUTTON,    NULL, 0, NULL, NULL,
				      XmVaCASCADEBUTTON, NULL, 0,
				      XmVaSEPARATOR,
				      XmVaPUSHBUTTON,    NULL, 0, NULL, NULL,
				      XmVaPUSHBUTTON,    NULL, 0, NULL, NULL,
				      XmVaSEPARATOR,
				      XmVaPUSHBUTTON,    NULL, 0, NULL, NULL,
				      NULL);
  smenu = XmVaCreateSimplePulldownMenu(menu, "EraseMenu",
				       1, erase_handler,
				       XmVaPUSHBUTTON, NULL, 0, NULL, NULL,
				       XmVaPUSHBUTTON, NULL, 0, NULL, NULL,
				       NULL);
  menu = XmVaCreateSimplePulldownMenu(menu_bar, "ActionsMenu",
				      1, actions_handler,
				      XmVaPUSHBUTTON, NULL, 0, NULL, NULL,
				      XmVaPUSHBUTTON, NULL, 0, NULL, NULL,
				      XmVaPUSHBUTTON, NULL, 0, NULL, NULL,
				      NULL);
  menu = XmVaCreateSimplePulldownMenu(menu_bar, "OptionsMenu",
				      2, options_handler,
				      XmVaPUSHBUTTON, NULL, 0, NULL, NULL,
				      XmVaPUSHBUTTON, NULL, 0, NULL, NULL,
				      XmVaPUSHBUTTON, NULL, 0, NULL, NULL,
				      NULL);

  menu = XmVaCreateSimplePulldownMenu(menu_bar, "HelpMenu",
				      3, help_handler,
				      XmVaPUSHBUTTON, NULL, 0, NULL, NULL,
				      XmVaPUSHBUTTON, NULL, 0, NULL, NULL,
				      XmVaSEPARATOR,
				      XmVaPUSHBUTTON, NULL, 0, NULL, NULL,
				      NULL, NULL);

  button = XmCreateCascadeButton(menu_bar, "HelpButton", NULL, 0);
  XtVaSetValues(button, XmNsubMenuId, menu, NULL, NULL);
  XtManageChild(button);
  XtVaSetValues(menu_bar, XmNmenuHelpWidget, button, NULL);

  widget = menu_bar;
} // Menus:: Menus()














